%% Run quasi-continuous third-order SMC with HOSMO to estimate x_1 and x_2

% Model parameter values

m = 2; % Kg
c = 5; % N/ms*2
k = 2; % N/m

% Observer gains
L_SO = 16;
k_1 = 6*L_SO^(1/3);
k_2 = 11*L_SO^(1/2);
k_3 = 6*L_SO;

% Sliding surface
c_1 = 1;

% Differentiator parameters
L_D = 1; % 
lambda_2 = 3;
lambda_1 = 1.5;
lambda_0 = 1.1;

% Control paramters

beta_1 = 1;
beta_2 = 2;

alpha = 30;

sim('MSDS_3_quasi_SMC_HOSMO_est_x_1_2');

%% Plot

figure(1)
subplot(4,1,1);
plot(x_1_des.time,x_1_des.signals.values,'r','LineSmoothing','on')
hold on
plot(x_1.time,x_1.signals.values,'b','LineSmoothing','on')
title('X_{des} with X'); xlabel('Time [s]');ylabel('Position of mass [m]');
legend('x_{des}','x');
xlim([0 50])

subplot(4,1,2);
plot(disturbance.time,disturbance.signals.values,'b','LineSmoothing','on')
title('Disturbance'); xlabel('Time [s]');ylabel('Force [N]');
xlim([0 50])

subplot(4,1,3);
plot(control_input.time,control_input.signals.values,'b','LineSmoothing','on')
title('Control input u'); xlabel('Time [s]');ylabel('Force [N]');
xlim([0 50])

subplot(4,1,4);
plot(sliding_var.time,sliding_var.signals.values,'b','LineSmoothing','on')
title('Sliding variable \sigma'); xlabel('Time [s]');
xlim([0 50])

figure(2)
subplot(4,1,1);
plot(x_1.time,x_1.signals.values-x_1_des.signals.values,'b','LineSmoothing','on')
title('Position error'); xlabel('Time [s]');ylabel('[m]');
xlim([0 50])

subplot(4,1,2);
plot(e_1.time,e_1.signals.values,'b','LineSmoothing','on')
title('Observer error e_1'); xlabel('Time [s]');ylabel('[m]');
xlim([0 50])

subplot(4,1,3);
plot(e_2.time,e_2.signals.values,'b','LineSmoothing','on')
title('Observer error e_2'); xlabel('Time [s]');ylabel('[m/s]');
xlim([0 50])

subplot(4,1,4);
plot(z_0.time,z_0.signals.values,'r','LineSmoothing','on')
hold on
plot(z_1.time,z_1.signals.values,'b','LineSmoothing','on')
hold on
plot(z_2.time,z_2.signals.values,'g','LineSmoothing','on')
title('Differentiator error z_0, z_1 and z_2'); xlabel('Time [s]');
legend('z_0','z_1','z_2');
xlim([0 50])
