function [tau_d, u_CM_2_dot,x_estimat_dot,differentiator_dot] = calculate_desired_forces_moments(t, theta,theta_dot, p_CM, p_CM_dot,u_CM_2,x_estimat,differentiator)
% Simple PD controller that returns the desired generalized forces and moments


global n p_CM_desired p_CM_dot_desired heading_desired heading_dot_desired t_vector ...
    sigma_f F_CM error_pos_CM p_CM_d e_1 e_2

%Paramters:
m_t = 16*0.6597; %total mass

if t <= 0
    tau_d = [0;0;0];
    u_CM_2_dot = zeros(2,1);
    x_estimat_dot = zeros(6,1); % x_1_dot = (1:2,1), x_2_dot = (3:4,1), x_3_dot = (5:6,1);
    differentiator_dot = zeros(6,1); % z_1_dot = (1:2,1), z_2_dot = (3:4,1), z_3_dot = (5:6,1); Differentiator 
end

%Choose controller: con = 1 (super-twisting), con = 2 (Nested third-order
%SMC), con = 3 (Quasi-continuous third-order SMC), con = 4 (PD-controller)
con = 1;

%Choose state observer: estimat = 0 (No state observer)
%estimat = 1 (State observer, estimate velocity), 
%estimat = 2 (Full state observer, estimate velocity and position).
estimat = 1; 

%Choose mode: torpedo mode = 1, operation mode = 2.
mode = 1;

% Calculates the desired thrust vector in 3 DOF (transport mode)
if t > 0
    %Control paramters
    F_CM = zeros(2,1);
    
    Kp_theta = 6;
    
    %Only for super-twisting
    u_CM_1 = zeros(2,1);
    u_CM_2_dot = zeros(2,1);
    STA_CM = zeros(2,1);
    
    %Only for third-order SMC
    third_C = zeros(2,1);
    u_1 = zeros(2,1);
    u_2 = zeros(2,1);
    
    %State observer
    z_1 = zeros(2,1);
    z_2 = zeros(2,1);
    z_3 = zeros(2,1);
    e_1 = zeros(2,1);
    e_2 = zeros(2,1);
    x_estimat_dot = zeros(6,1);
    
    % Differentiator 
    v_0 = zeros(2,1);
    v_1 = zeros(2,1);
    v_2 = zeros(2,1);
    differentiator_dot = zeros(6,1);
    
    if mode == 1
        % Tordpedo mode
        Delta=2.24;
        heading_d = -atan2(p_CM(2,1),Delta);
    else
        % Operation mode
        if (t <= 10) 
            heading_d = sin(pi/2);
        end
        if (t > 10 && t <= 20) || (t > 30 && t <= 40)
            heading_d = sin(pi);
        end
        if (t > 20 && t <= 30)  
            heading_d = sin(3*pi/2);
        end
    end   
    
    p_CM_dot_d = interp1(t_vector,p_CM_dot_desired',t,'spline')';
    p_CM_d = interp1(t_vector,p_CM_desired',t,'spline')';
    
    heading = theta(n);
    
    %Observer dynamics 
    if estimat ~= 0
        L_SO = 50;
        k_1 = 6*L_SO^(1/3);
        k_2 = 11*L_SO^(1/2);
        k_3 = 6*L_SO;
        
        e_1 = p_CM - x_estimat(1:2,1);
        e_2 = p_CM_dot - x_estimat(3:4,1);
        
        for i = 1:2
            z_1(i) = k_1*abs(e_1(i))^(2/3)*sign(e_1(i));
            z_2(i) = k_2*abs(e_1(i))^(1/3)*sign(e_1(i));
            z_3(i) = k_3*sign(e_1(i));
        end
        
        x_estimat_dot(5:6,1) = z_3;
        x_estimat_dot(3:4,1) = x_estimat(5:6,1) + z_2 + (1/m_t)*(F_CM); 
        x_estimat_dot(1:2,1) = x_estimat(3:4,1) + z_1;
    end
    
    % Error variables
    error_pos_CM = p_CM - p_CM_d;
    error_vel_CM = p_CM_dot - p_CM_dot_d;
    error_heading = heading_d - heading;
    
    error_pos_CM_estimat = x_estimat(1:2,1) - p_CM_d;
    error_vel_CM_estimat = x_estimat(3:4,1) - p_CM_dot_d;
    
    
    % Create sliding surfaces
    c_1 = 1;    
    if estimat == 0
        sigma_f = error_vel_CM + error_pos_CM;
    elseif estimat == 1
        sigma_f = error_vel_CM_estimat + error_pos_CM;
    else
        sigma_f = error_vel_CM_estimat + c_1*error_pos_CM_estimat;
    end
    
    %Differentiator dynamics
    if (con == 2) || (con == 3)
        if con == 2
            L_D = 0.1; 
        else 
            L_D = 1;
        end
        lambda_0 = 1.1;
        lambda_1 = 1.5;
        lambda_2 = 3;
        
        z_0 = differentiator(1:2,1);
        z_1 = differentiator(3:4,1);
        z_2 = differentiator(5:6,1);
       
        for i = 1:2
            v_0(i) = -lambda_2*(L_D)^(1/3)*(abs(z_0(i)-sigma_f(i)))^(2/3)*sign(z_0(i)-sigma_f(i))+z_1(i);
            v_1(i) = -lambda_1*(L_D)^(1/2)*(abs(z_1(i)-v_0(i)))^(1/2)*sign(z_1(i)-v_0(i))+z_2(i);
            v_2(i) = -lambda_0*L_D*sign(z_2(i)-v_1(i));
        end
        
        differentiator_dot(5:6,1) = v_2;
        differentiator_dot(3:4,1) = v_1;
        differentiator_dot(1:2,1) = v_0;
     end
    
    if con == 1 %Super-Twisting controller
        K_f = 10;

        for i = 1:2
            u_CM_1(i) = -1.5*sqrt(K_f)*sqrt(abs(sigma_f(i)))*sign(sigma_f(i));
            u_CM_2_dot(i) = -1.1*K_f*sign(sigma_f(i));
            STA_CM(i) = u_CM_1(i) + u_CM_2(i);
        end
        u_C = STA_CM;
        
    elseif  con == 2 %Nested third-order SMC
        alpha = 10;
        beta_2 = 1;
        beta_1 = 1;
        
        z_0 = differentiator(1:2,1);
        z_1 = differentiator(3:4,1);
        z_2 = differentiator(5:6,1);
        
        for i = 1:2
            third_C(i) = -alpha*(sign(z_2(i)+beta_2*((abs(z_1(i)))^3+(abs(z_0(i)))^2)^(1/6)*sign(z_1(i)+beta_1*((abs(z_0(i)))^(2/3)*sign(z_0(i))))));
        end
        
        u_C = third_C;
 
    elseif  con == 3 %Quasi-continuous third-order SMC
        alpha = 10;
        beta_2 = 2;
        beta_1 = 1;
        
        z_0 = differentiator(1:2,1);
        z_1 = differentiator(3:4,1);
        z_2 = differentiator(5:6,1);
        
        for i = 1:2
            u_1(i) = z_2(i) + beta_2*(abs(z_1(i))+abs(z_0(i))^(2/3))^(-1/2)*(z_1(i)+beta_1*abs(z_0(i))^(2/3)*sign(z_0(i)));
            u_2(i) = abs(z_2(i))+beta_2*(abs(z_1(i))+abs(z_0(i))^(2/3))^(1/2);

            third_C(i)= -alpha*(u_1(i)/u_2(i));
        end
        if t < 0.01
            u_C = zeros(2,1);
        else
            u_C = third_C;
        end
        
    else %PD-controller
        Kp_pos = 10; %0.6
        Kd_pos = 0.1; %0.06;
       
        if estimat == 0
            u_C = Kd_pos*(-1)*error_vel_CM + Kp_pos*(-1)*error_pos_CM;
        elseif estimat == 1
            u_C = Kd_pos*(-1)*error_vel_CM_estimat + Kp_pos*(-1)*error_pos_CM;
        else
            u_C = Kd_pos*(-1)*error_vel_CM_estimat + Kp_pos*(-1)*error_pos_CM_estimat;
        end
    end
    if estimat == 0
        F_CM = m_t*(u_C);
    elseif estimat == 1
        if con == 3
            if t < 0.01
                F_CM = zeros(2,1);
            else
                F_CM = (u_C - c_1*x_estimat(3:4,1) - x_estimat(5:6,1) - z_2);
            end
        else
            F_CM = m_t*(u_C - c_1*x_estimat(3:4,1) - x_estimat(5:6,1) - z_2);
        end
    else
        if con == 3
            if t < 0.01
                F_CM = zeros(2,1);
            else
                F_CM = (u_C - c_1*x_estimat(3:4,1) - x_estimat(5:6,1) - z_2 - c_1*z_1);
            end
        else
            F_CM = m_t*(u_C - c_1*x_estimat(3:4,1) - x_estimat(5:6,1) - z_2 - c_1*z_1);
        end
    end 
    T_CM = min(Kp_theta*error_heading,100);
    tau_d = [F_CM; T_CM];
end

end
