function [phi_dot_dot, theta_dot_dot, p_CM_dot_dot] = calculate_q_dot_dot_CM(theta_dot, fx, fy, u, fTx, fTy)
%
% Calculates and returns the acceleration of theta and p_CM and the constraint joint forces
% for each link of the snake robot. Theta are the absolute link angles and
% p_CM is the position of the center of mass of the snake.
%

global n l J D e N  V  V_x_a V_y_a m...
sin_theta_diag cos_theta_diag  mi_n  lambda1 lambda2 lambda3


% Create variables for common expressions
T1=sin_theta_diag*N*mi_n*sin_theta_diag;
T2=cos_theta_diag*N*mi_n*cos_theta_diag;
T3=cos_theta_diag*N*mi_n*sin_theta_diag;
T4=cos_theta_diag*N'*sin_theta_diag;
T5=cos_theta_diag*N'*cos_theta_diag;
T6=(e'*mi_n*sin_theta_diag)*sin_theta_diag;
T7=(e'*mi_n*sin_theta_diag)*cos_theta_diag;
T8=(e'*mi_n*cos_theta_diag)*cos_theta_diag;
T9=l*N'*sin_theta_diag;
T10=l*N'*cos_theta_diag;


%Calculate useful matrixes

Mtot=[n*m+T6*e -T7*e;...
    -T7*e n*m+T8*e];

Mtot_inv=inv(Mtot);

M11=Mtot_inv(1,1); M12=Mtot_inv(1,2); M21=Mtot_inv(2,1); M22=Mtot_inv(2,2);



N1=l*T1*(sin_theta_diag*e)+l*T3*(cos_theta_diag*e);
N2=l*T1*(cos_theta_diag*e)+l*T2*(cos_theta_diag*e);

K1=(-M11*T6+M12*T7)*T9...
    -(M11*T7-M12*T8)*T10;
 
K2=(-M11*T6+M12*T7)*T10...
    +(M11*T7-M12*T8)*T9;

K3=(M11*T7-M12*T8)*V_x_a...
    +(M11*T6-M12*T7)*V_y_a;

K4=(-M21*T6+M22*T7)*T9...
    -(M21*T7-M22*T8)*T10;
 

K5=(-M21*T6+M22*T7)*T10...
    +(M21*T7-M22*T8)*T9;

K6=(M21*T7-M22*T8)*V_x_a...
    +(M21*T6-M22*T7)*V_y_a;

% Calculates the inertia matrix

M=J*eye(n)+m*l^2*sin_theta_diag*V*sin_theta_diag+m*l^2*cos_theta_diag*V*cos_theta_diag+lambda1*eye(n)...
    +l^2*T1*sin_theta_diag*N'*sin_theta_diag...
    +l^2*T1*T5...
    +l^2*T3*T4...
    +l^2*T2*T5...
    +N1*K1-N2*K4;

% Calculates various other matrices

W = m*l^2*sin_theta_diag*V*cos_theta_diag-m*l^2*cos_theta_diag*V*sin_theta_diag...
    +l^2*T1*sin_theta_diag*N'*cos_theta_diag...
    -l^2*T1*T4...
    +l^2*T3*T5...
    -l^2*T2*T4...
    +N1*K2-N2*K5;

V_theta=lambda2*eye(n)-l*T1*cos_theta_diag*V_x_a...
    -l*T1*sin_theta_diag*V_y_a...
    -l*T2*cos_theta_diag*V_x_a...
    -l*T3*cos_theta_diag*V_y_a...
    +N1*K3-N2*K6;
    

% Calculates a vector of squared angular link velocities
theta_dot_sq = zeros(n, 1);
theta_dot_abs = zeros(n, 1);

for i = 1:n
    theta_dot_sq(i, 1) = theta_dot(i, 1)*theta_dot(i, 1);
    theta_dot_abs(i, 1) = theta_dot(i, 1)*abs(theta_dot(i, 1));
end

% Constructs the inertia matrix of the system

% M_inv=inv(M);  % Not needed

% Calculates a matrix related to Coriolis and centripetal forces
W_th=W*theta_dot_sq+V_theta*theta_dot+lambda3*eye(n)*theta_dot_abs...
    +(N1*(M11*e')-N2*(M21*e')-l*sin_theta_diag*N)*fx+(N1*(M12*e')-N2*(M22*e')+l*cos_theta_diag*N)*fy;

% Calculates a matrix related to the thruster forces
H_thruster = l*sin_theta_diag*(N*fTx) - l*cos_theta_diag*(N*fTy);

% Calculates the theta_dot_dot
theta_dot_dot = M\(D'*u-W_th+H_thruster);


px_dot_dot=K1*theta_dot_dot+K2*theta_dot_sq+K3*theta_dot+(M11*e')*fx+(M12*e')*fy + (M11*e')*fTx+(M12*e')*fTy;
py_dot_dot=K4*theta_dot_dot+K5*theta_dot_sq+K6*theta_dot+(M21*e')*fx+(M22*e')*fy + (M21*e')*fTx+(M22*e')*fTy;

p_CM_dot_dot=[px_dot_dot;py_dot_dot];

% Calculates the joint angle acceleration
phi_dot_dot=D*theta_dot_dot;