function [u, phi_ref] = calculate_u_lateral_undulation(t, phi, phi_dot,theta,p_CM, p_CM_dot)
%
% Calculates and returns the actuator forces. This is the joint controller
% of the snake robot.
%

global n Kp_joint Kd_joint alpha omega delta heading phi_offset heading_ref

k_psi = 1*0.8;

%Choose mode: torpedo mode = 1, operation mode = 2
mode = 1;

if mode == 1
    % Tordpedo mode
    Delta=2.24;
    heading_ref = -atan2(p_CM(2,1),Delta);
else
    % Operation mode
    if (t <= 10)
        heading_ref = sin(pi/2);
    end
    if (t > 10 && t <= 20) || (t > 30 && t <= 40)
        heading_ref = sin(pi);
    end
    if (t > 20 && t <= 30)
        heading_ref = sin(3*pi/2);
    end
end

% Here you choose whether you want the undulation pattern to follow a
% line-of-sight reference heading, and if you want undulation or not
heading = theta(n); 
phi_offset = k_psi * (heading_ref - heading);

if mode == 1
    alpha = 0;  % No lateral undulation
end

% Calculates references for joint angles.
phi_ref = zeros(n-1, 1);
for i = 1:n-1
    phi_ref(i, 1) = alpha*sin(omega*t + (i-1)*delta) - phi_offset;
end

% Calculates references for joint velocities.
phi_ref_d = zeros(n-1, 1);
for i = 1:n-1
    phi_ref_d(i, 1) = alpha*omega*cos(omega*t + (i-1)*delta);
end
% Calculates references for joint accelerations.
phi_ref_dd = zeros(n-1, 1);
for i = 1:n-1
    phi_ref_dd(i, 1) = -alpha*omega^2*sin(omega*t + (i-1)*delta);
end

% Calculates the actuator forces.
u = zeros(n-1, 1);
for i = 1:n-1
    error = phi_ref(i, 1) - phi(i, 1);
    error_d = phi_ref_d(i, 1) - phi_dot(i, 1);
    %u(i, 1) = Kp_joint*error - Kd_joint*phi_dot(i, 1);
    u(i, 1) = phi_ref_dd(i, 1) + Kd_joint*error_d + Kp_joint*error;
end
