function v_dot = calculate_v_dot(t, v)
%
% Calculates and returns v_dot.
%
% v = [theta ; p_CM ; theta_dot ; p_CM_dot ; u_CM_2 ; x_estimat]
%

global W_thr_inv link_number thruster_axis_configuration n D sin_theta_diag cos_theta_diag...
      friction_model ...
    previous_draw_time visualize_motion timestep ...
    store_sim_data sim_data_save_interval previous_simdata_save ...
    t_sim z_sim z_dot_sim phi phi_ref phi_ref_sim phi_sim f_thr_sim tau_d_sim T_allocation_sim...
    fTx_sim fTy_sim fx_sim fy_sim thrusterAmplitude heading phi_offset heading_ref orientation_ref_sim...
    orientation_sim px_sim py_sim phi_offset_sim ...
    sigma_f F_CM error_pos_CM p_CM_d sigma_f_sim F_CM_sim error_pos_CM_sim p_CM_d_sim ...
    e_1 e_2 observer_error_e_1_sim observer_error_e_2_sim differentiator_error


% Extracts the states from the state vector
theta               = v(1:n, 1);
p_CM                = v(n+1:n+2, 1);
theta_dot           = v(n+3:2*n+2, 1);
p_CM_dot            = v(2*n+3:2*n+4, 1);
u_CM_2              = v(2*n+5:2*n+6,1);
x_estimat(1:2,1)    = v(2*n+7:2*n+8,1);
x_estimat(3:4,1)    = v(2*n+9:2*n+10,1);
x_estimat(5:6,1)    = v(2*n+11:2*n+12,1);
differentiator(1:6,1)            = v(2*n+13:2*n+18,1);
%z(3:4,1)            = v(2*n+15:2*n+16,1);
%z(5:6,1)            = v(2*n+17:2*n+18,1);

% Calculates diagonal matrices that are used in the proceeding calculations
sin_theta_diag = calculate_sin_theta_diag(theta);
cos_theta_diag = calculate_cos_theta_diag(theta);


% Calculates the joint angles
phi = D*theta;
% Calculates the joint angle velocities
phi_dot = D*theta_dot;

% Calculates the position of the links
z = calculate_z(theta, [], p_CM);

% Calculates the linear velocities of the links
z_dot = calculate_z_dot(theta_dot, [], p_CM_dot);

% Calculates translational friction forces.
fx = zeros(n, 1);
fy = zeros(n, 1);
if strcmp(friction_model, 'MORISON')
    [fx, fy] = calculate_f_drag(theta_dot, z_dot);
elseif strcmp(friction_model, 'VISCOUS')
    [fx, fy, fx_debug] = calculate_f_viscous(t, theta, theta_dot, p_CM_dot, z_dot);
end

% Calculates rotational friction forces (friction torques).
tau = zeros(n, 1);
%tau = calculate_tau_coulomb(theta_dot, z_dot);

% Calculates joint torques based on the desired joint motion, i.e. lateral
% undulation or eel-like undulation. Can be replaced with your own code to
% generate a random joint motion disturbance.

%[u, phi_ref] = calculate_u_eel_undulation(t, phi, phi_dot, theta, p_CM, p_CM_dot);
[u, phi_ref] = calculate_u_lateral_undulation(t, phi, phi_dot, theta, p_CM, p_CM_dot);


% Calculates thruster allocation matrix
[bX, bY, BX, BY] = calculate_thruster_config_matrices_3DOF(link_number, thruster_axis_configuration, theta, n);
T_allocation_3DOF = calculate_allocation_matrix_3DOF(bX, bY, BX, BY, sin_theta_diag, cos_theta_diag);

% Calculates the global frame forces and moments in 2D. Can be replaced
% with your own code.
[tau_d, u_CM_2_dot, x_estimat_dot,differentiator_dot] = calculate_desired_forces_moments(t, theta, theta_dot, p_CM, p_CM_dot, u_CM_2, x_estimat,differentiator);

% Distributes the thruster efforts among the thrusters based on the global
% force/moment input tau_d
[f_thr] = calculate_thruster_forces(tau_d, T_allocation_3DOF, W_thr_inv);



% Decomposes the scalar thruster forces in the global X and Y directions
fTx = BX'*f_thr;
fTy = BY'*f_thr;


% Calculates the acceleration of theta and p where p is the position of the CM
[phi_dot_dot, theta_dot_dot, p_CM_dot_dot] = calculate_q_dot_dot_CM(theta_dot, fx, fy, u, fTx, fTy);

% figure(1000)
% plot(theta_dot)
% pause(0.01)
% Constructs the derivative of the state vector
v_dot = [theta_dot ; p_CM_dot ; theta_dot_dot ; p_CM_dot_dot ; u_CM_2_dot; x_estimat_dot;differentiator_dot];

% Visualizes the snake robot
if visualize_motion
    % Draws the snake if the specified time step has passed
    if t >= previous_draw_time + timestep*10
        % Updates the time of the previously drawn position
        previous_draw_time = t;
        % Draws the next position
        drawSnakeRobot(t, theta, p_CM, 1, z, fTx, fTy);
        % Gives Matlab time to show the drawn graph
        pause(timestep);
        %pause(0.2);
    end
end

% Stores simulation data for debug purposes.
if store_sim_data
    % Clears stored simulation data.
    if t <= 0
        previous_simdata_save = 0;
        t_sim = [];
        z_sim = [];
        z_dot_sim = [];
        phi_sim = [];
        phi_ref_sim = [];
        f_thr_sim = [];
        fTx_sim = [];
        fTy_sim = [];
        fx_sim = [];
        fy_sim = [];
        orientation_ref_sim = [];
        orientation_sim = [];
        px_sim = [];
        py_sim = [];
        phi_offset_sim = [];
        sigma_f_sim = [];
        F_CM_sim = [];
        error_pos_CM_sim = [];
        p_CM_d_sim = [];
        observer_error_e_1_sim = [];
        observer_error_e_2_sim = [];
        differentiator_error = [];
        
        %tau_d_sim = [];
        %T_allocation_sim = [];
    end
    
    % Stores simulation data for debug purposes.
    if t >= previous_simdata_save + sim_data_save_interval
        previous_simdata_save = t;
        t_sim(end+1, 1) = t;
        z_sim(end+1, :) = z';
        z_dot_sim(end+1, :) = z_dot';
        phi_sim(end+1, :)=phi;
        phi_ref_sim(end+1, :)=phi_ref;
        f_thr_sim(end+1, :) = f_thr;
        fTx_sim(end+1, :) = fTx;
        fTy_sim(end+1, :) = fTy;
        fx_sim(end+1, :) = fx;
        fy_sim(end+1, :) = fy;
        orientation_ref_sim(end+1, :) = heading_ref;
        orientation_sim(end+1, :) = heading;
        px_sim(end+1, :) = p_CM(1,1);
        py_sim(end+1, :) = p_CM(2,1);
        phi_offset_sim(end+1, :) = phi_offset; 
        sigma_f_sim(end+1, :) = sigma_f;
        F_CM_sim(end+1, :) = F_CM;
        error_pos_CM_sim(end+1, :) = error_pos_CM;
        p_CM_d_sim(end+1, :) = p_CM_d;
        observer_error_e_1_sim(end+1,:) = e_1;
        observer_error_e_2_sim(end+1,:) = e_2;
        differentiator_error(end+1,:) = differentiator;
        %tau_d_sim(end+1, :) = tau_d;
        %T_allocation_sim(end+1, :, :) = T_allocation_3DOF;
    end
end
