global W_thruster W_thr_inv link_number thruster_axis_configuration n g m_tot m l J A D N l_N l_N_tr...
    e E R theta0 theta0_dot p_CM0 u_CM0_2 x_estimat_dot z_dot friction_model ...
    store_sim_data sim_data_save_interval visualizeCMposition visualizeThrusterForces...
    visualize_motion timestep fignumber ...
    ct cn mi_n lambda1 lambda2 lambda3 N_N_tr V V_x_a V_y_a V_current ...
    save_to_AVI AVI_file_name AVI_fps AVI_speedfactor AVI_include_axes M_temp idx...
    alpha delta thrusterAmplitude

disp('Initializing model parameters...')

%=========================================================================
% Thruster configuration
%=========================================================================

thruster_configuration = 1;

if thruster_configuration == 1
    link_number = [1 3 6 11 14]; 
    thruster_axis_configuration = ['X' 'Y' 'Y' 'Y' 'Y'];
    W_thruster = diag([1 1 1 1 1]); 
    W_thr_inv = diag(1./diag(W_thruster));
end

if thruster_configuration == 2
    link_number = [1 3 14]; 
    thruster_axis_configuration = ['X' 'Y' 'Y'];
    W_thruster = diag([1 1 1]); 
    W_thr_inv = diag(1./diag(W_thruster));
end



%=========================================================================
% Ground friction model
%=========================================================================

friction_model = 'MORISON';

% Gravitational constant.
g = 9.81;


%=========================================================================
% Kinematic and dynamic parameters
%=========================================================================

% Number of links. The system has n+2 DOFs.
% Temporarily disabled.
n = 16;

% The mass of each link
m =0.6597;

% The length of each link is 2*l
l = 0.07;

% The moment of inertia of each link
J = (m*l^2)/3;

%--m_tot-----------------
m_tot = n*m;

%Parameter for eliptical joint
a=0.05;
b=0.03;
ro=1000;
C_f=0.03;
C_d=2;
C_A=1;
C_M=1;

%Parameter for drag forces
ct=0.25;   %1/2*ro*pi*C_f*(a+b)/2*2*l;
cn=1;      %1/2*ro*C_d*2*b*2*l;
%Parameter for added mass forces
mi_n=pi*ro*b^2*C_A*2*l;
 %Parameter for added mass torque
lambda1=1/12*ro*pi*C_M*(b^2-a^2)^2*l^3;
 %Parameter for linear drag torque
lambda2=1/6*ro*pi*C_f*(a+b)*l^3;
 %Parameter for nonlinear drag torque
lambda3=1/8*ro*pi*C_f*(a+b)*l^4;

% Addition matrix
A = [];
for i = 1:n-1
    for j = 1:n
        if (i==j)|(i==j-1) 
            A(i,j) = 1;
        end
    end
end

% Difference matrix
D = [];
for i = 1:n-1
    for j = 1:n
        if (i==j)
            D(i,j) =  1;
        elseif (i==j-1)
            D(i,j) = -1;
        end
    end
end

% Summation vector
e = ones(n,1);
E = [e          zeros(n,1);
     zeros(n,1) e         ];

R = [];
for i = 1:n
    R(i, :) = [zeros(1, i-1) ones(1, n-i+1)];
end

N=A'*inv(D*D')*D;
l_N = l*N;
l_N_tr = l_N';
N_N_tr=N*N';
V=A'*inv(D*D')*A;

%Current effect
v_x=0;
v_y=0;
V_x=v_x*ones(n,1);
V_y=v_y*ones(n,1);
V_current=[V_x;V_y];
V_x_a=v_x*eye(n,n);
V_y_a=v_y*eye(n,n);

%=========================================================================
% Initial values
%=========================================================================y
theta0 = zeros(n, 1); %ones(n,1)*pi/2;  %alpha*sin(((1:n)'-1)*delta);  
theta0_dot = zeros(n, 1);
phi0 = zeros(n, 1); %alpha*sin(((1:n)'-1)*delta);  
phi0_dot = zeros(n, 1);
p_CM0 = [0 ; 0];
p_CM0_dot = [0 ; 0];
u_CM0_2 = [0 ; 0];
x_estimat_dot = zeros(6,1);
z_dot = zeros(6,1);

%=========================================================================
% Constructing reference path and heading
%=========================================================================

createReferencePathAndHeading;

% Reference angles for DP mode
% maxJointAngle = 15;
% pp_x = 0:1:40;
% pp_x = repmat(pp_x,n-1,1);
% pp_y = -maxJointAngle*(pi/180) + 2*maxJointAngle*(pi/180)*randn(size(pp_x));
% pp = cell(n-1,1);
% for i = 1:n-1
%     pp{i} = spline(pp_x(i,:),pp_y(i,:));
%     phi_ref_vector(i,:) = ppval(pp{i},t_vector);
%     phi_ref_d_vector(i,:) = diff(phi_ref_vector(i,:))/h;
% end

idx = 1;


%=========================================================================
% Visualization parameters
%=========================================================================

% Indicates if the motion should be visualized during a simulation
visualize_motion = 1;

% Indicates if the CM of the snake robot should be visualized
visualizeCMposition = 1;

% Indicates if the amplitude and direction of thruster forces should be
% visualized
visualizeThrusterForces = 1;

% Time step for the visualization
timestep = 0.01;

% The number of the figure window to be used for motion visualization
fignumber = 100;

% Indicates if simulation data should be stored.
store_sim_data = 1;

% The time step between each save.
sim_data_save_interval = 0.001;


%=========================================================================
% AVI file creation parameters
%=========================================================================

% Indicates if an AVI-file should be created from the motion displayed
% by visualizeMotion.m
save_to_AVI = 1;

% The name of the AVI file
AVI_file_name = 'USM';

% Number of frames per second
AVI_fps = 30;

% Speed multiplier = 1 gives real-time playback
AVI_speedfactor = 1;

% Indicates if the axes should be visible in the AVI file or if only the
% area inside the graph should be visualized
AVI_include_axes = 1;
