%
% Starts the snake robot simulation.
%

global visualize_motion previous_draw_time t_previous waypoints ...
    WP_switch_times t_sim theta_pathframe_sim p_pathframe_sim ...
    vt_sim psi_path_sim z_sim z_dot_sim f_thr_sim tau_d_sim  ...
    T_allocation_sim phi_sim phi_ref_sim...
    fTx_sim fTy_sim fx_sim fy_sim orientation_ref_sim...
    orientation_sim px_sim py_sim phi_offset_sim ...
    sigma_f_sim F_CM_sim error_pos_CM_sim p_CM_d_sim ...
    observer_error_e_1_sim observer_error_e_2_sim differentiator_error


% The stop time for the simulation.
stop_time = 40; % Straight motion
%stop_time = 200; % Turning motion

% Initializes controller parameters.
initControllerParameters;

% Initializes model parameters.
initModelParameters;


disp('Starting simulation...')

% Matlab function used to measure how long the simulation takes
tic

% The simulation time of the previous time step
t_previous = 0;

% Constructs the initial values for the state vector.
v0 = [theta0 ; p_CM0 ; theta0_dot ; p_CM0_dot ; u_CM0_2 ; x_estimat_dot; z_dot;];

% Starts the snake robot visualization
if visualize_motion
    % The time of the previously drawn sample
    previous_draw_time = 0;

    % Draws the initial position of the snake robot
    drawSnakeRobot(0, theta0, p_CM0, 0);
    
    % Create a slider with callback function to change \omega
%     f33 = figure(33);
%     set(f33,'Position', [800 600 130 30]);
%     sld = uicontrol('Style', 'slider',...
%         'Min',20,'Max',100,'Value',70,...
%         'Position', [0 0 120 20],...
%         'Callback', @changeOmega); 
    
    % Waits for the figure window to display
    pause(0.1);
end
%return

if 1

ode = 5;
% ode23tb solver
if ode == 23
    options = odeset('Events', 'off', 'RelTol', 6e-2, 'AbsTol', 6e-2); 
    [t, v] = ode23tb('calculate_v_dot', [0 stop_time], v0, options);
end

% ode5 solver 
if ode == 5
    nstep = 4000000;
    tspan = linspace(0,stop_time,nstep);
    v = ode5('calculate_v_dot',tspan,v0);
end

disp('Simulation complete...')

% Displays how long the simulation took to complete
toc

disp(['Simulated time: ' num2str(t(end, 1)) 's'])

% Extracts the states from the state vector.
theta       = v(:, 1:n);
p_CM        = v(:, n+1:n+2);
theta_dot   = v(:, n+3:2*n+2);
p_CM_dot    = v(:, 2*n+3:2*n+4);
%phi         = v(:, 2*n+5:3*n+4);
%phi_dot     = v(:, 3*n+5:4*n+4);
%phi_o       = v(:, 4*n+5);
%psi_ref     = v(:, 4*n+8);

end
%return